package gui;

import java.applet.Applet;
import java.awt.*;

public final class Evt {

    private Evt() {
    }

    public static boolean matchKey(Event e, int key) {
        return ((e.id == Event.KEY_PRESS) && (e.key == key));
    }

    private static boolean matchButton(Event e, Button target) {
        return
                ((e.id == Event.ACTION_EVENT) && e.target.equals(target));
    }


    public static boolean match(Event e, int key, String str) {
        return (matchKey(e, key) || e.arg.equals(str));
    }

    // checking order is important
    public static boolean match(Event e, int key, Object target) {
        if (e.target instanceof MenuItem)
            return e.target.equals(target);
        switch (e.id) {
            case Event.KEY_PRESS:
                return matchKey(e, key);
            case Event.ACTION_EVENT:
                return matchButton(e, (Button) target);
            default:
                return false;
        }
    }

    public static boolean match(Event e, String str) {
        return (e.arg.equals(str));
    }

    public static boolean match(Event e, Object target) {
        int c = getKeyboardShortCut(e, target);
        return match(e, c, target);
    }

    public static int getKeyboardShortCut(Event e, Object target) {
        if (target instanceof MenuItem) {
            MenuItem mi = (MenuItem) target;
            String str = mi.getLabel();
            int index = str.indexOf('[');
            if (index == 0) {
                return str.charAt(1);
            }
        }
        return -1;
    }

    /**
      Return true if arg of Event.arg is an applet name
      otherwise return false.
    */
    public static boolean applet(Event e) {
        Applet a;
        String appletName;
        Class classForArg;
        if (e.arg instanceof Applet) {
            System.out.println("Evt.applet got an applet");
            return true;
        }
        if (!(e.arg instanceof String))
            return false;
        appletName = (String) e.arg;
        //System.out.println("Starting applet:"+appletName);
        try {
            classForArg = Class.forName(appletName);
        } catch (ClassNotFoundException ex) {
            //System.out.println("Evt:ClassNotFoundException:"+ex);
            return false;
        } catch (Exception ex) {
            //System.out.println("Exception:"+ex);
            return false;
        }
        try {
            // create an instance of your applet class
            a = (Applet) classForArg.newInstance();
        } catch (InstantiationException ex) {
            //System.out.println("Evt:InstantiationException:"+ex);
            return false;
        } catch (IllegalAccessException ex) {
            //System.out.println("Evt:IllegalAccessException:"+ex);
            return false;
        } catch (IllegalArgumentException ex) {
            //System.out.println("IllegalArgumentException:" + ex);
            return false;
        }
        return true;
    }


}