package ipl;

import vs.ColorUtils;
import vs.ImageUtils;
import vs.FFT;

import java.awt.*;


public class FFTPlane {

    private PixelPlane pp;

    private ColorUtils CU = new ColorUtils();
    private ImageUtils iUtil = new ImageUtils();
    private FFT imgFFT = new FFT();


    // Standard Java ARGB pixel.
    private int[] imageData;
    private Image img;

    private short[] imageData_R;
    private short[] imageData_G;
    private short[] imageData_B;

    public FFTPlane(ProcessPlane pp_) {
        imgFFT.DisplayLogPSD = true;

        pp = pp_;
        img = pp.makeImage();


        imageData = pp.pels;
        int w = pp.getWidth();
        int h = pp.getHeight();
        // separated R,G,B values
        imageData_R = CU.getRedArray(imageData);
        imageData_G = CU.getGreenArray(imageData);
        imageData_B = CU.getBlueArray(imageData);
    }

    public void threshold(double t) {
        float[][] redR = imgFFT.getRedReal();
        float[][] redI = imgFFT.getRedImaginary();
        float[][] greenR = imgFFT.getGreenReal();
        float[][] greenI = imgFFT.getGreenImaginary();
        float[][] blueR = imgFFT.getBlueReal();
        float[][] blueI = imgFFT.getBlueImaginary();

        // multiply by scalefactor, sf, to
        // make filter normalized from 0..1.
        float sf = 1 / 255f;

        float r, g, b;

        for (int x = 0; x < pp.getWidth(); x++) {
            for (int y = 0; y < pp.getHeight(); y++) {
                if (mag(x, y) < t) {
                    redR[x][y] = 0;
                    redI[x][y] = 0;
                    greenR[x][y] = 0;
                    greenI[x][y] = 0;
                    blueR[x][y] = 0;
                    blueI[x][y] = 0;
                }
            }
        }
        // update the pp with the new value for the psd.
        System.out.println("FFTPlane: computing psd...");
        pp.pels = imgFFT.psd();
    }

    private float mag(int x, int y) {
        float[][] redR = imgFFT.getRedReal();
        float[][] redI = imgFFT.getRedImaginary();
        float[][] greenR = imgFFT.getGreenReal();
        float[][] greenI = imgFFT.getGreenImaginary();
        float[][] blueR = imgFFT.getBlueReal();
        float[][] blueI = imgFFT.getBlueImaginary();
        float rr = redR[x][y];
        float ri = redI[x][y];
        float gr = greenR[x][y];
        float gi = greenI[x][y];
        float br = blueR[x][y];
        float bi = blueI[x][y];

        return (float) Math.sqrt(
                (rr * rr +
                ri * ri +
                gr * gr +
                gi * gi +
                br * br +
                bi * bi) / 6.0);

    }

    public void mult(ProcessPlane ppIn) {
        float[][] redR = imgFFT.getRedReal();
        float[][] redI = imgFFT.getRedImaginary();
        float[][] greenR = imgFFT.getGreenReal();
        float[][] greenI = imgFFT.getGreenImaginary();
        float[][] blueR = imgFFT.getBlueReal();
        float[][] blueI = imgFFT.getBlueImaginary();

        // multiply by scalefactor, sf, to
        // make filter normalized from 0..1.
        float sf = 1 / 255f;

        float r, g, b;

        for (int x = 0; x < pp.getWidth(); x++) {
            for (int y = 0; y < pp.getHeight(); y++) {
                r = ppIn.getRed(x, y) * sf;
                g = ppIn.getGreen(x, y) * sf;
                b = ppIn.getBlue(x, y) * sf;
                redR[x][y] *= r;
                redI[x][y] *= r;
                greenR[x][y] *= g;
                greenI[x][y] *= g;
                blueR[x][y] *= b;
                blueI[x][y] *= b;
            }
        }
        // update the pp with the new value for the psd.
        System.out.println("FFTPlane: computing psd...");
        pp.pels = imgFFT.psd();
    }

    //----------------------------------------------------------------
    //
    //
    //----------------------------------------------------------------
    public void fft() {
        // Make sure an image exists.
        int w = pp.getWidth();
        int h = pp.getHeight();
        if ((h == w) &&
                (h != 0)) {
            // Forward FFT on input image data in short arrays,
            // the complex data is returned in cX_r & cX_i arrays
            imageData = imgFFT.forward2dFFT(imageData_R, imageData_G,
                    imageData_B, w, h);
            pp.pels = imageData;
        } else
            System.out.println("ER!:The input image must be 2^n*2^n");
    }

    public void ifft() {
        // Get the FFT array data that was calculated.
        float[][] cR_r = imgFFT.getRedReal();
        float[][] cR_i = imgFFT.getRedImaginary();
        float[][] cG_r = imgFFT.getGreenReal();
        float[][] cG_i = imgFFT.getGreenImaginary();
        float[][] cB_r = imgFFT.getBlueReal();
        float[][] cB_i = imgFFT.getBlueImaginary();
        int w = pp.getWidth();
        int h = pp.getHeight();

        // Make sure a complex representation of an image
        // has at least been allocated.
        if ((cR_r != null) && (cR_i != null) &&
                (cG_r != null) && (cG_i != null) &&
                (cB_r != null) && (cB_i != null)) {
            // Do the reverse FFT
            imageData = imgFFT.reverse2dFFT();
            pp.pels = imageData;
        } else
            System.out.println("ER! FFTPlane: null ifft detected");
    }

}
