package observers;

import java.awt.*;
import java.util.Observable;
import java.util.Observer;

//
// IntTextField - a TextField that reads in integer values and
// updates an Observable int with the new value.  This class
// is both a "view" of the Observable int, since it displays
// its current value, and a "controller" since it updates the
// value.

public class StringTextField extends TextField implements Observer {
    private ObservableString stringValue;

    public StringTextField(ObservableInt theString) {
// Initialize the field to the current value, allow 3 input columns
        super("" + theString.getValue(), 3);
        //stringValue = theString;
        stringValue.addObserver(this);     // Express interest in value
    }
// The action for the text field is called whenever someone presses "return"
// We'll try to convert the string in the field to an integer, and if
// successful, update the observable int.
    public boolean action(Event evt, Object whatAction) {
        String str;          // to be converted from a string
        try {     // The conversion can throw an exception
            str = getText();
// If we get here, there was no exception, update the observable
            stringValue.setValue(str);
        } catch (Exception oops) {
// We just ignore the exception
        }
        return true;
    }
// The update action is called whenever the observable int's value changes.
// We just update the text in the field with the new int value
    public void update(Observable obs, Object arg) {
        setText("" + stringValue.getValue());
    }
}