package solid;

import diffcad.point;

public class Mat4 {
    double a[] [];
    public static final int N = 4;

    private static double piOn180 = Math.PI / 180.0;

    public static Mat4 translation(point t) {
        double m[][] = new double[N][N];
        m[0][0] = 1;
        m[1][1] = 1;
        m[2][2] = 1;
        m[0][2] = t.x;
        m[1][2] = t.y;
        return new Mat4(m);
    }

    public static Mat4 scaling(point s) {
        double m[][] = new double[N][N];
        m[0][0] = s.x;
        m[1][1] = s.y;
        m[2][2] = 1;
        return new Mat4(m);
    }

    public static Mat4 shear(point sh) {
        double m[][] = new double[N][N];
        m[0][0] = 1;
        m[1][1] = 1;
        m[2][2] = 1;
        m[0][1] = sh.x;
        m[1][0] = sh.y;
        return new Mat4(m);
    }

    public static Mat4 rotation(double theta) {
        double m[][] = new double[N][N];
        double cas = Math.cos(theta);
        double sas = Math.sin(theta);
        m[0][0] = cas;
        m[1][1] = cas;
        m[0][1] = -sas;
        m[1][0] = sas;
        return new Mat4(m);
    }

    public static void main(String args[]) {
        Mat4 trans1 = Mat4.shear(new point(1, 2));
        trans1.print();
    }

    public Mat4(double a_[][]) {
        a = a_;
    }

    public double[][] getArray() {
        return a;
    }

/*
To generate a time-optimal 4x4 matrix inversion, use maple:
with(linalg):readlib(C):
a:=array(0..3,0..3,[]):
 b:=array(0..3,0..3,[]):
 b:=inverse(matrix(a)):
 C(b,optimized);

*/
    public Mat4 invert() {
        double b[] [] = new double[N][N];
        double t14 = a[0][0] * a[1][1];
        double t15 = a[2][2] * a[3][3];
        double t17 = a[2][3] * a[3][2];
        double t19 = a[0][0] * a[2][1];
        double t20 = a[1][2] * a[3][3];
        double t22 = a[1][3] * a[3][2];
        double t24 = a[0][0] * a[3][1];
        double t25 = a[1][2] * a[2][3];
        double t27 = a[1][3] * a[2][2];
        double t29 = a[1][0] * a[0][1];
        double t32 = a[1][0] * a[2][1];
        double t33 = a[0][2] * a[3][3];
        double t35 = a[0][3] * a[3][2];
        double t37 = a[1][0] * a[3][1];
        double t38 = a[0][2] * a[2][3];
        double t40 = a[0][3] * a[2][2];
        double t42 = t14 * t15 - t14 * t17 - t19 * t20 + t19 * t22 + t24 * t25 - t24 * t27 - t29 * t15 + t29 * t17 + t32
                * t33 - t32 * t35 - t37 * t38 + t37 * t40;
        double t43 = a[2][0] * a[0][1];
        double t46 = a[2][0] * a[1][1];
        double t49 = a[2][0] * a[3][1];
        double t50 = a[0][2] * a[1][3];
        double t52 = a[0][3] * a[1][2];
        double t54 = a[3][0] * a[0][1];
        double t57 = a[3][0] * a[1][1];
        double t60 = a[3][0] * a[2][1];
        double t63 = t43 * t20 - t43 * t22 - t46 * t33 + t46 * t35 + t49 * t50 - t49 * t52 - t54 * t25 + t54 * t27 + t57
                * t38 - t57 * t40 - t60 * t50 + t60 * t52;
        double t65 = 1 / (t42 + t63);
        double t71 = a[0][2] * a[2][1];
        double t73 = a[0][3] * a[2][1];
        double t75 = a[0][2] * a[3][1];
        double t77 = a[0][3] * a[3][1];
        double t81 = a[0][1] * a[1][2];
        double t83 = a[0][1] * a[1][3];
        double t85 = a[0][2] * a[1][1];
        double t87 = a[0][3] * a[1][1];
        double t101 = a[1][0] * a[2][2];
        double t103 = a[1][0] * a[2][3];
        double t105 = a[2][0] * a[1][2];
        double t107 = a[2][0] * a[1][3];
        double t109 = a[3][0] * a[1][2];
        double t111 = a[3][0] * a[1][3];
        double t115 = a[0][0] * a[2][2];
        double t117 = a[0][0] * a[2][3];
        double t119 = a[2][0] * a[0][2];
        double t121 = a[2][0] * a[0][3];
        double t123 = a[3][0] * a[0][2];
        double t125 = a[3][0] * a[0][3];
        double t129 = a[0][0] * a[1][2];
        double t131 = a[0][0] * a[1][3];
        double t133 = a[1][0] * a[0][2];
        double t135 = a[1][0] * a[0][3];
        b[0][0] = (a[1][1] * a[2][2] * a[3][3] - a[1][1] * a[2][3] * a[3][2] - a[2][1] * a[1]
                [2] * a[3][3] + a[2][1] * a[1][3] * a[3][2] + a[3][1] * a[1][2] * a[2][3] - a[3][1] * a[1][3] * a
                [2][2]) * t65;
        b[0][1] = -(a[0][1] * a[2][2] * a[3][3] - a[0][1] * a[2][3] * a[3][2] - t71 * a[3][3] +
                t73 * a[3][2] + t75 * a[2][3] - t77 * a[2][2]) * t65;
        b[0][2] = (t81 * a[3][3] - t83 * a[3][2] - t85 * a[3][3] + t87 * a[3][2] + t75 * a[1][3] -
                t77 * a[1][2]) * t65;
        b[0][3] = -(t81 * a[2][3] - t83 * a[2][2] - t85 * a[2][3] + t87 * a[2][2] + t71 * a[1][3] -
                t73 * a[1][2]) * t65;
        b[1][0] = -(t101 * a[3][3] - t103 * a[3][2] - t105 * a[3][3] + t107 * a[3][2] + t109 * a[2]
                [3] - t111 * a[2][2]) * t65;
        b[1][1] = (t115 * a[3][3] - t117 * a[3][2] - t119 * a[3][3] + t121 * a[3][2] + t123 * a[2]
                [3] - t125 * a[2][2]) * t65;
        b[1][2] = -(t129 * a[3][3] - t131 * a[3][2] - t133 * a[3][3] + t135 * a[3][2] + t123 * a[1]
                [3] - t125 * a[1][2]) * t65;
        b[1][3] = (t129 * a[2][3] - t131 * a[2][2] - t133 * a[2][3] + t135 * a[2][2] + t119 * a[1]
                [3] - t121 * a[1][2]) * t65;
        b[2][0] = (t32 * a[3][3] - t103 * a[3][1] - t46 * a[3][3] + t107 * a[3][1] + t57 * a[2][3] -
                t111 * a[2][1]) * t65;
        b[2][1] = -(t19 * a[3][3] - t117 * a[3][1] - t43 * a[3][3] + t121 * a[3][1] + t54 * a[2][3]
                - t125 * a[2][1]) * t65;
        b[2][2] = (t14 * a[3][3] - t131 * a[3][1] - t29 * a[3][3] + t135 * a[3][1] + t54 * a[1][3] -
                t125 * a[1][1]) * t65;
        b[2][3] = -(t14 * a[2][3] - t131 * a[2][1] - t29 * a[2][3] + t135 * a[2][1] + t43 * a[1][3]
                - t121 * a[1][1]) * t65;
        b[3][0] = -(t32 * a[3][2] - t101 * a[3][1] - t46 * a[3][2] + t105 * a[3][1] + t57 * a[2][2]
                - t109 * a[2][1]) * t65;
        b[3][1] = (t19 * a[3][2] - t115 * a[3][1] - t43 * a[3][2] + t119 * a[3][1] + t54 * a[2][2] -
                t123 * a[2][1]) * t65;
        b[3][2] = -(t14 * a[3][2] - t129 * a[3][1] - t29 * a[3][2] + t133 * a[3][1] + t54 * a[1][2]
                - t123 * a[1][1]) * t65;
        b[3][3] = (t14 * a[2][2] - t129 * a[2][1] - t29 * a[2][2] + t133 * a[2][1] + t43 * a[1][2] -
                t119 * a[1][1]) * t65;

        return new Mat4(b);
    }
/* To generate a new matrix, transposed:
 with(linalg):readlib(C):
 a:=array(0..3,0..3,[]):
 b:=array(0..3,0..3,[]):
 b:=transpose(matrix(a)):
 C(b,optimized);
*/
    public Mat4 transpose() {
        double b[][] = new double[3][3];
        b[0][0] = a[0][0];
        b[0][1] = a[1][0];
        b[0][2] = a[2][0];
        b[0][3] = a[3][0];
        b[1][0] = a[0][1];
        b[1][1] = a[1][1];
        b[1][2] = a[2][1];
        b[1][3] = a[3][1];
        b[2][0] = a[0][2];
        b[2][1] = a[1][2];
        b[2][2] = a[2][2];
        b[2][3] = a[3][2];
        b[3][0] = a[0][3];
        b[3][1] = a[1][3];
        b[3][2] = a[2][3];
        b[3][3] = a[3][3];
        return new Mat4(b);
    }

// To optimize the matrix multiplication
// for the non-sparse matrix, use:
/*
 a:=array(0..3,0..3,[]):
 b:=array(0..3,0..3,[]):
 c:=array(0..3,0..3,[]):
 am := matrix(a):bm:=matrix(b):x:=matrix(c):
 x := multiply(bm, am):
 C(x,optimized);

*/
// You may need to reformat the output with
// an emacs macro....just to get rid of the indendation.

    public Mat4 multiply(Mat4 bmat4) {
        double x [][] = new double[N][N];
        double b [][] = bmat4.getArray();
        x[0][0] = b[0][0] * a[0][0] + b[0][1] * a[1][0] + b[0][2] * a[2][0] + b[0][3] * a[3][0];
        x[0][1] = b[0][0] * a[0][1] + b[0][1] * a[1][1] + b[0][2] * a[2][1] + b[0][3] * a[3][1];
        x[0][2] = b[0][0] * a[0][2] + b[0][1] * a[1][2] + b[0][2] * a[2][2] + b[0][3] * a[3][2];
        x[0][3] = b[0][0] * a[0][3] + b[0][1] * a[1][3] + b[0][2] * a[2][3] + b[0][3] * a[3][3];
        x[1][0] = b[1][0] * a[0][0] + b[1][1] * a[1][0] + b[1][2] * a[2][0] + b[1][3] * a[3][0];
        x[1][1] = b[1][0] * a[0][1] + b[1][1] * a[1][1] + b[1][2] * a[2][1] + b[1][3] * a[3][1];
        x[1][2] = b[1][0] * a[0][2] + b[1][1] * a[1][2] + b[1][2] * a[2][2] + b[1][3] * a[3][2];
        x[1][3] = b[1][0] * a[0][3] + b[1][1] * a[1][3] + b[1][2] * a[2][3] + b[1][3] * a[3][3];
        x[2][0] = b[2][0] * a[0][0] + b[2][1] * a[1][0] + b[2][2] * a[2][0] + b[2][3] * a[3][0];
        x[2][1] = b[2][0] * a[0][1] + b[2][1] * a[1][1] + b[2][2] * a[2][1] + b[2][3] * a[3][1];
        x[2][2] = b[2][0] * a[0][2] + b[2][1] * a[1][2] + b[2][2] * a[2][2] + b[2][3] * a[3][2];
        x[2][3] = b[2][0] * a[0][3] + b[2][1] * a[1][3] + b[2][2] * a[2][3] + b[2][3] * a[3][3];
        x[3][0] = b[3][0] * a[0][0] + b[3][1] * a[1][0] + b[3][2] * a[2][0] + b[3][3] * a[3][0];
        x[3][1] = b[3][0] * a[0][1] + b[3][1] * a[1][1] + b[3][2] * a[2][1] + b[3][3] * a[3][1];
        x[3][2] = b[3][0] * a[0][2] + b[3][1] * a[1][2] + b[3][2] * a[2][2] + b[3][3] * a[3][2];
        x[3][3] = b[3][0] * a[0][3] + b[3][1] * a[1][3] + b[3][2] * a[2][3] + b[3][3] * a[3][3];

        return (new Mat4(x));
    }

/*
In maple:
x:=array(0..3,[]):
vec4:=vector([v1, v2, v3, v4]):
x:=multiply(matrix(a),vec4):
C(x,optimized):
does a post-multiplication, A*v
*/
    public double[] multiply(double v1, double v2, double v3, double v4) {
        double x[] = new double[N];
        x[0] = a[0][0] * v1 + a[0][1] * v2 + a[0][2] * v3 + a[0][3] * v4;
        x[1] = a[1][0] * v1 + a[1][1] * v2 + a[1][2] * v3 + a[1][3] * v4;
        x[2] = a[2][0] * v1 + a[2][1] * v2 + a[2][2] * v3 + a[2][3] * v4;
        x[3] = a[3][0] * v1 + a[3][1] * v2 + a[3][2] * v3 + a[3][3] * v4;
        return x;
    }
/*
To do a pre-multiplication, v*A
vec4:=vector([v1, v2, v3, v4]):
 C(multiply(matrix(a),vec4),optimized):
*/
    public double[] premultiply(double v1, double v2, double v3) {
        double x[] = new double[N];
        x[0] = v1 * a[0][0] + a[0][1] * v2 + a[0][2] * v3;
        x[1] = a[1][0] * v1 + v2 * a[1][1] + a[1][2] * v3;
        x[2] = a[2][0] * v1 + a[2][1] * v2 + v3 * a[2][2];
        return x;
    }

    public void print() {
        for (int i = 0; i < N; i++) {
            for (int j = 0; j < N; j++)
                System.out.print(a[i][j] + " ");
            System.out.println();
        }
    }

}